import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Button, notification, message, Modal } from 'antd'
import moment from 'moment'
import md5 from 'md5'

import Api from '@/api'
import Utils from '@/utils/utils.js'
import MKEmitter from '@/utils/events.js'
import MkIcon from '@/components/mk-icon'

// import './index.scss'
const { confirm } = Modal

class FuncButton extends Component {
  static propTpyes = {
    BID: PropTypes.string,
    btn: PropTypes.object,
    selectedData: PropTypes.any,
    disabled: PropTypes.any
  }

  state = {
    loading: false,
    disabled: false,
    hidden: false,
    dict: window.GLOB.dict
  }

  UNSAFE_componentWillMount () {
    const { btn, selectedData, BData, disabled } = this.props

    if (btn.controlField) {
      this.setStatus(btn, selectedData || [], BData, disabled)
    } else if (disabled) {
      this.setState({disabled: true})
    }
  }

  componentDidMount () {
    MKEmitter.addListener('triggerBtnId', this.actionTrigger)
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.state), fromJS(nextState))
  }

  UNSAFE_componentWillReceiveProps (nextProps) {
    const { btn } = this.props

    if (btn.controlField) {
      this.setStatus(btn, nextProps.selectedData || [], nextProps.BData, nextProps.disabled)
    } else {
      this.setState({disabled: nextProps.disabled === true})
    }
  }

  componentWillUnmount () {
    this.setState = () => {
      return
    }
    MKEmitter.removeListener('triggerBtnId', this.actionTrigger)
  }

  setStatus = (btn, data, BData, disprop) => {
    let disabled = false
    let hidden = false

    if (btn.control !== 'parent') {
      if (data.length > 0) {
        data.forEach(item => {
          let s = item[btn.controlField] !== undefined ? item[btn.controlField] + '' : ''
          if (btn.controlVals.includes(s)) {
            disabled = true
          }
        })
      } else if (btn.controlVals.includes('')) {
        disabled = true
      }
    } else {
      if (!BData || !BData.hasOwnProperty(btn.controlField)) {
        hidden = true
      } else {
        let s = BData[btn.controlField] + ''
        if (btn.controlVals.includes(s)) {
          hidden = true
        }
      }
    }

    if (disabled && btn.control === 'hidden') {
      hidden = true
    }

    if (disprop) {
      disabled = true
    }

    this.setState({hidden, disabled})
  }

  /**
   * @description 触发按钮操作
   */
  actionTrigger = (triggerId, record, type, lid) => {
    const { btn, BID, selectedData, LID } = this.props
    const { loading, dict } = this.state

    if (loading) return
    if (triggerId && btn.uuid !== triggerId) return
    if (type === 'linkbtn' && !btn.$toolbtn && LID !== lid) return

    let data = record || selectedData || []
    let error = ''

    if (btn.funcType === 'shareLink' && window.GLOB.systemType === 'production' && !btn.shareProUrl) {
      error = dict['no_prod_link'] || '尚未设置正式系统链接地址！'
    } else if (btn.funcType === 'refund') {
      if (data.length === 0) {
        error = dict['select_row'] || '请选择行！'
      } else if (data.length !== 1) {
        error = dict['select_single_row'] || '请选择单行数据！'
      } else if (!data[0].$$uuid) {
        error = dict['no_ordercode'] || '未获取到订单编号！'
      }
    }

    if (error) {
      notification.warning({
        top: 92,
        message: error,
        duration: 5
      })
      return
    }

    if (btn.funcType === 'shareLink') {
      let bid = BID || ''
      let id = ''
      if (data[0]) {
        id = data[0].$$uuid || ''
      }
  
      let url = btn.shareUrl
      if (window.GLOB.systemType === 'production') {
        url = btn.shareProUrl
      }
  
      url = url.replace(/@BID@/ig, bid).replace(/@ID@/ig, id)
  
      if (btn.shortUrl === 'true') {
        this.setState({
          loading: true
        }, () => {
          this.getShortUrl(url)
        })
      } else {
        this.copyUrl(url)
      }
    } else if (btn.funcType === 'refund') {
      let orderId = data[0].$$uuid
      const that = this

      confirm({
        title: btn.tipTitle || dict['exec_sure'] || '确定要执行吗?',
        okText: dict['ok'] || '确定',
        cancelText: dict['cancel'] || '取消',
        onOk() {
          that.execRefund(orderId, data[0])
        },
        onCancel() {}
      })
    }
  }

  execRefund = (orderId, data) => {
    const { btn, BID } = this.props

    let param = null
    if (btn.payMode === 'inner') {
      param = {
        func: btn.innerFunc || '',
        BID: BID || '',
        username: sessionStorage.getItem('User_Name') || '',
        fullname: sessionStorage.getItem('Full_Name') || '',
        dataM: sessionStorage.getItem('dataM') === 'true' ? 'Y' : '',
        ID: orderId
      }
    } else if (window.backend && window.GLOB.CacheData.has('sql_' + btn.uuid)) {
      param = this.getbackParam(orderId, data)
    } else if (btn.payMode === 'system') {
      let sql = this.getSysDeclareSql(orderId, data)

      param = {
        func: 'sPC_TableData_InUpDe',
        BID: BID || '',
        exec_type: window.GLOB.execType || 'y',
        timestamp: moment().format('YYYY-MM-DD HH:mm:ss')
      }

      param.secretkey = Utils.encrypt('', param.timestamp)
      param.LText = Utils.formatOptions(sql, param.exec_type)

      if (btn.output) {
        param.key_back_type = 'Y'
      }

      if (window.GLOB.mkHS) { // 函数 sPC_TableData_InUpDe 云端验证
        param.open_key = Utils.encryptOpenKey(param.secretkey, param.timestamp)
      }

      param.menuname = btn.logLabel

      if (window.GLOB.probation) {
        param.s_debug_type = 'Y'
      }
    }

    this.setState({loading: true})

    if (param) {
      Api.genericInterface(param).then(res => {
        if (res.status) {
          let id = orderId
          if (btn.output) {
            id = res.mk_b_id || res[btn.output] || orderId
          }

          Api.setRefund({out_biz_no: id}).then(res => {
            if (!res.status) {
              this.execError({ErrCode: 'E', message: res.message || window.GLOB.dict['exc_fail'] || '执行失败！', ...res})
            } else {
              this.execSuccess({ErrCode: 'S', ...res})
            }
          })
        } else {
          this.execError(res)
        }
      })
    } else {
      Api.setRefund({out_biz_no: orderId}).then(res => {
        if (!res.status) {
          this.execError({ErrCode: 'E', message: res.message || window.GLOB.dict['exc_fail'] || '执行失败！', ...res})
        } else {
          this.execSuccess({ErrCode: 'S', ...res})
        }
      })
    }
  }

  getbackParam = (ID, data) => {
    const { columns, btn, BID } = this.props

    let ex = window.GLOB.CacheData.get('sql_' + btn.uuid)
    let exps = []
    let values = {
      time_id: Utils.getguid(),
      roleid: sessionStorage.getItem('role_id') || '',
      mk_departmentcode: sessionStorage.getItem('departmentcode') || '',
      mk_organization: sessionStorage.getItem('organization') || '',
      mk_user_type: sessionStorage.getItem('mk_user_type') || '',
      mk_nation: sessionStorage.getItem('nation') || '',
      mk_province: sessionStorage.getItem('province') || '',
      mk_city: sessionStorage.getItem('city') || '',
      mk_district: sessionStorage.getItem('district') || '',
      mk_address: sessionStorage.getItem('address') || '',
      id: ID || '',
      bid: BID || '',
      typename: 'admin',
      datam: sessionStorage.getItem('dataM') === 'true' ? 'Y' : '',
      datam_begin: sessionStorage.getItem('dataM') === 'true' ? 'Y' : '',
      datam_end: sessionStorage.getItem('dataM') === 'true' ? 'Y' : '',
    }
  
    if (window.GLOB.externalDatabase !== null) {
      values.db = window.GLOB.externalDatabase
    }

    ex.reps.forEach(n => {
      let key = n.toLowerCase()
      if (values.hasOwnProperty(key)) {
        exps.push({
          key: n,
          value: values[key]
        })
      }
    })

    let _data = {}
    Object.keys(data).forEach(key => {
      _data[key.toLowerCase()] = data[key]
    })
  
    columns.forEach(col => {
      if (!ex.reps.includes(col.field)) return
      if (!col.datatype) return

      let _key = col.field.toLowerCase()
      let _val = _data.hasOwnProperty(_key) ? _data[_key] : ''

      if (/^date/ig.test(col.datatype) && !_val) {
        _val = '1949-10-01'
      }

      exps.push({
        key: 'mk_' + col.field + '_mk',
        value: _val
      })
    })

    let md5_id = ''
    if (window.GLOB.probation) {
      md5_id = md5(ex.id + JSON.stringify(exps) + Math.floor(new Date().getTime() / 600000))
      md5_id = moment().format('YYYYMMDDHHmmss') + md5_id.slice(-18)
    }

    return {
      $backend: true,
      $type: 's_TableData_InUpDe',
      data: [{
        id: ex.id,
        menuname: btn.logLabel || '',
        exps: exps,
        md5_id: md5_id
      }]
    }
  }

  getSysDeclareSql = (ID, data) => {
    const { columns, btn, BID } = this.props

    // 系统变量
    let _vars = ['tbid', 'errorcode', 'retmsg', 'username', 'fullname', 'roleid', 'mk_departmentcode', 'mk_organization', 'mk_user_type', 'mk_nation', 'mk_province', 'mk_city', 'mk_district', 'mk_address', 'bid']
    let _declare = []
    let _initVal = []
  
    let _data = {}
    Object.keys(data).forEach(key => {
      _data[key.toLowerCase()] = data[key]
    })
  
    columns.forEach(col => {
      let _key = col.field.toLowerCase()

      if (_vars.includes(_key)) return

      _declare.push(`@${_key} ${col.datatype}`)

      let _val = _data.hasOwnProperty(_key) ? _data[_key] : ''

      if (/^date/ig.test(col.datatype) && !_val) {
        _val = '1949-10-01'
      }

      if (/'/.test(_val)) {
        _val = _val.replace(/'/ig, '"')
      }

      _initVal.push(`@${_key}='${_val}'`)
    })
  
    // 变量声明
    _declare = _declare.join(',')
    if (_declare) {
      _declare = ',' + _declare
    }

    let _sql = `/* 系统生成 */
      Declare @ErrorCode nvarchar(50),@retmsg nvarchar(4000), @UserName nvarchar(50),@FullName nvarchar(50),@RoleID nvarchar(512),@mk_departmentcode nvarchar(512),@mk_organization nvarchar(512),@mk_user_type nvarchar(20),@mk_nation nvarchar(50),@mk_province nvarchar(50),@mk_city nvarchar(50),@mk_district nvarchar(50),@mk_address nvarchar(100),@bid nvarchar(50),@tbid nvarchar(50)${_declare}
    `
  
    let userName = sessionStorage.getItem('User_Name') || ''
    let fullName = sessionStorage.getItem('Full_Name') || ''
    let RoleID = sessionStorage.getItem('role_id') || ''
    let departmentcode = sessionStorage.getItem('departmentcode') || ''
    let organization = sessionStorage.getItem('organization') || ''
    let mk_user_type = sessionStorage.getItem('mk_user_type') || ''
    let nation = sessionStorage.getItem('nation') || ''
    let province = sessionStorage.getItem('province') || ''
    let city = sessionStorage.getItem('city') || ''
    let district = sessionStorage.getItem('district') || ''
    let address = sessionStorage.getItem('address') || ''
  
    // 初始化凭证及用户信息字段
    _sql += `
      /* 用户信息初始化赋值 */
      select @ErrorCode='',@retmsg='',@UserName='${userName}', @FullName='${fullName}', @RoleID='${RoleID}', @mk_departmentcode='${departmentcode}', @mk_organization='${organization}', @mk_user_type='${mk_user_type}', @mk_nation='${nation}', @mk_province='${province}', @mk_city='${city}', @mk_district='${district}', @mk_address='${address}', @bid='${BID}'
    `
    _sql += `
      /* 显示列变量赋值 */
      select ${_initVal.join(',')}
    `

    btn.verify.scripts.forEach(item => {
      if (item.status === 'false') return

      _sql += `
      ${item.sql}
      `
    })

    if (btn.output) {
      _sql += `
        aaa: select @ErrorCode as ErrorCode,@retmsg as retmsg,${btn.output} as mk_b_id`
    } else {
      _sql += `
        aaa: select @ErrorCode as ErrorCode,@retmsg as retmsg`
    }

    _sql = _sql.replace(/@ID@/ig, `'${ID || ''}'`)
    _sql = _sql.replace(/@BID@/ig, `'${BID || ''}'`)
    _sql = _sql.replace(/@LoginUID@/ig, `'${sessionStorage.getItem('LoginUID') || ''}'`)
    _sql = _sql.replace(/@SessionUid@/ig, `'${localStorage.getItem('SessionUid') || ''}'`)
    _sql = _sql.replace(/@UserID@/ig, `'${sessionStorage.getItem('UserID') || ''}'`)
    _sql = _sql.replace(/@Appkey@/ig, `'${window.GLOB.appkey || ''}'`)
    _sql = _sql.replace(/@lang@/ig, `'${sessionStorage.getItem('lang')}'`)
    _sql = _sql.replace(/@typename@/ig, `'admin'`)

    if (window.GLOB.externalDatabase !== null) {
      _sql = _sql.replace(/@db@/ig, window.GLOB.externalDatabase)
    }

    if (sessionStorage.getItem('dataM') === 'true') { // 数据权限
      _sql = _sql.replace(/\$@/ig, '/*').replace(/@\$/ig, '*/').replace(/@datam@/ig, `'Y'`)
    } else {
      _sql = _sql.replace(/@\$|\$@/ig, '').replace(/@datam@/ig, `''`)
    }

    if (window.GLOB.debugger === true) {
      window.mkInfo('%c' + btn.logLabel, 'color: blue')
      window.mkInfo(_sql)
    }

    return _sql
  }

  getShortUrl = (url) => {
    let _rduri = window.atob('aHR0cHM6Ly9lcGMubWs5aC5$mkjbi93ZWJhcGkvZG9zdGFycw=='.replace(/\$mk/ig, ''))
    let _id = window.atob('YmgwYmFwYWJ0ZDQ1ZXBz$mkZ3JhNzlzZWdiY2g2YzFpYms='.replace(/\$mk/ig, ''))

    let param = {
      func: 's_url_db_adduptdel',
      appkey: window.GLOB.appkey,
      userid: _id,
      LoginUID: _id,
      type: 'add_only',
      validity: 15,
      linkurl: url,
      nonc: '' + new Date().getTime(),
      id: md5(url + window.GLOB.appkey)
    }

    let keys = Object.keys(param).sort()
    let values = ''
    keys.forEach(key => {
      values += key + param[key]
    })
    param.sign = md5(values)
    param.t = new Date().getTime()

    Api.directRequest({
      url: _rduri + '/s_url_db_adduptdel',
      method: 'post',
      data: JSON.stringify(param)
    }).then(res => {
      this.setState({
        loading: false
      })

      if (res.status && res.id) {
        this.copyUrl('https://mk9h.cn/m.asp?m=' + res.id)
      } else {
        notification.warning({
          top: 92,
          message: res.message || '链接生成失败！',
          duration: 5
        })
      }
    }, () => {
      this.setState({
        loading: false
      })
    })
  }

  copyUrl = (url) => {
    const { btn } = this.props

    let oInput = document.createElement('input')
    oInput.value = url
    document.body.appendChild(oInput)
    oInput.select()
    document.execCommand('Copy')
    document.body.removeChild(oInput)

    if (btn.shareTip) {
      Modal.success({
        title: btn.shareTip
      })
    } else {
      message.success(window.GLOB.dict['copied'] || '已复制到剪切板。')
    }
  }

  execSuccess = (res = {}) => {
    const { btn } = this.props
    const { dict } = this.state

    if (res.ErrCode === 'S' || !res.ErrCode) { // 执行成功
      notification.success({
        top: 92,
        message: res.message || dict['exc_success'] || '执行成功！',
        duration: btn.verify && btn.verify.stime ? btn.verify.stime : 2
      })
    } else if (res.ErrCode === 'Y') { // 执行成功
      Modal.success({
        title: res.message || dict['exc_success'] || '执行成功！',
        okText: dict['got_it'] || '知道了'
      })
    } else if (res.ErrCode === '-1') { // 完成后不提示

    }

    this.setState({
      loading: false
    })

    if (btn.execSuccess !== 'never') {
      MKEmitter.emit('refreshByButtonResult', btn.$menuId, btn.execSuccess, btn)
    }
  }

  execError = (res) => {
    const { btn } = this.props
    const { dict } = this.state

    if (!['LoginError', 'C', '-2', 'E', 'N', 'F', 'NM'].includes(res.ErrCode)) {
      res.ErrCode = 'E'
    }

    if (res.ErrCode === 'E') {
      Modal.error({
        title: res.message || dict['exc_fail'] || '执行失败！',
        okText: dict['got_it'] || '知道了'
      })
    } else if (res.ErrCode === 'N') {
      notification.error({
        top: 92,
        message: res.message || dict['exc_fail'] || '执行失败！',
        duration: btn.verify && btn.verify.ntime ? btn.verify.ntime : 10
      })
    } else if (res.ErrCode === 'F') {
      notification.error({
        className: 'notification-custom-error',
        top: 92,
        message: res.message || dict['exc_fail'] || '执行失败！',
        duration: btn.verify && btn.verify.ftime ? btn.verify.ftime : 10
      })
    } else if (res.ErrCode === 'NM') {
      message.error(res.message || dict['exc_fail'] || '执行失败！')
    }
    
    this.setState({
      loading: false
    })

    if (res.ErrCode === '-2') return

    if (btn.execError !== 'never') {
      MKEmitter.emit('refreshByButtonResult', btn.$menuId, btn.execError, btn)
    }
  }

  render() {
    const { btn } = this.props
    const { loading, hidden } = this.state

    if (hidden) return null

    let label = ''

    if (btn.show === 'link') {
      label = <span>{btn.label}{btn.icon ? <MkIcon style={{marginLeft: '8px'}} type={btn.icon} /> : ''}</span>
    } else if (btn.show === 'icon') {
      label = !loading ? <MkIcon type={btn.icon} /> : null
    } else {
      label = <span>{!loading && btn.icon ? <MkIcon style={{marginRight: '8px'}} type={btn.icon} /> : ''}{btn.label}</span>
    }
    
    return (
      <Button
        type="link"
        title={btn.show === 'icon' ? btn.label : ''}
        loading={loading}
        style={btn.style || null}
        className={btn.hover || ''}
        onClick={(e) => {e.stopPropagation(); this.actionTrigger()}}
      >{label}</Button>
    )
  }
}

export default FuncButton